/* #define RCVR_CLEAN_CORE_DUMP */
/*+-----------------------------------------------------------------------
	ecusighdl.c - xmtr/rcvr individual process signal handlers
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	child_signals()
	kill_rcvr_process(sig)
	rcvr_SIGHUP_handler()
	rcvr_SIGINT_handler()
	rcvr_SIGTERM_handler()
	rcvr_SIGUSR1_handler()
	rcvr_SIGUSR2_handler()
	rcvr_common_signal_handler()
	rcvr_death_handler(sig)
	rcvr_signals()
	start_rcvr_process(notify_flag)
	termecu(sig)
	termecu_code_text(code)
	xmtr_SIGCLD_handler()
	xmtr_SIGHUP_handler(sig)
	xmtr_SIGINT_handler()
	xmtr_SIGTERM_handler(sig)
	xmtr_SIGUSR2_handler()
	xmtr_death_handler(sig)
	xmtr_signals()

------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-25-1991-23:56-wht@n4hgf2-handle xmtr core dump gracefully */
/*:07-25-1991-12:56-wht@n4hgf-ECU release 3.10 */
/*:07-17-1991-07:04-wht@n4hgf-avoid SCO UNIX nap bug */
/*:06-29-1991-15:42-wht@n4hgf-if WHT and xterm, play with title bar */
/*:01-29-1991-12:57-wht@n4hgf-on exit, restore setcolor colors if possible */
/*:12-18-1990-20:02-wht@n4hgf-add rcvr_death_handler */
/*:09-19-1990-19:36-wht@n4hgf-ecu_log_event now gets pid for log from caller */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include "ecufork.h"

extern int errno;
extern int rcvr_pid;
extern int xmtr_pid;
extern int windows_active;
extern int current_ttymode;
extern int ttymode_termecu_on_sigint;
extern char lopen_err_str[];
extern int rcvr_log;
extern FILE *rcvr_log_fp;
extern char rcvr_log_file[];	/* if rcvr_log!= 0,log filename */
extern int rcvr_log_append;

int sigint = 0;			/* interrupt indicator */
int proc_interrupt = 0;		/* procedure interrupt indicator */
int last_child_wait_status = 0;
int last_child_wait_pid = 0;
int xmtr_killed_rcvr = 0;

char *signal_name_text();

void termecu();
void xmtr_signals();

void rcvr_SIGTERM_handler();
void rcvr_SIGINT_handler();
void rcvr_SIGUSR1_handler();
void rcvr_SIGUSR2_handler();
void rcvr_SIGHUP_handler();
void xmtr_SIGINT_handler();
void xmtr_SIGHUP_handler();
void xmtr_SIGTERM_handler();
void xmtr_SIGCLD_handler();
void Xcvr_death_handler();
void rcvr_common_signal_handler();
void rcvr_death_handler();

/*+-----------------------------------------------------------------------
	start_rcvr_process(notify_flag)
------------------------------------------------------------------------*/
void
start_rcvr_process(notify_flag)
int notify_flag;
{
#if defined(FORK_DEBUG)
char s40[40];
#endif
extern ulong colors_current;
ulong colors_at_entry = colors_current;

	fflush(so);
	fflush(se);

	if(rcvr_pid > 0)			/* if process already active,just ... */
		return;

	if(rcvr_log && rcvr_log_file[0] && rcvr_log_fp)
	{
		fclose(rcvr_log_fp);
		rcvr_log_fp = NULL;
	}

	xmtr_killed_rcvr = 0;
	rcvr_pid = smart_fork();
	if(rcvr_pid == 0)		/* if we are the (spawned) rcvr process */
	{
		if(notify_flag)
		{
			setcolor(colors_notify);
			fputs("[interactive mode]",se);
			setcolor(colors_at_entry);
			fputs("\r\n",se);
		}

		rcvr();	/* run until killed */
		/*NOTREACHED*/
	}
	else if(rcvr_pid > 0) 	/* we are the father (xmtr) process */
	{
#if defined(FORK_DEBUG)
		sprintf(s40,"DEBUG rcvr pid %d",rcvr_pid);
		ecu_log_event(getpid(),s40);		/* rcvr */
#endif
		if(rcvr_log)
			rcvr_log_append = 1;	/* until next %log -s */
		xmtr_signals();
		return;
	}
	rcvr_pid = -1;		/* no receiver active */
	ff(se,"\r\n\ncould not fork for receive\r\n");
	termecu(TERMECU_NO_FORK_FOR_RCVR);
	/*NOTREACHED*/

}	/* end of start_rcvr_process */

/*+-----------------------------------------------------------------------
	kill_rcvr_process(sig) -- kill rcvr process with signal 'sig'
------------------------------------------------------------------------*/
void
kill_rcvr_process(sig)
int sig;
{
int wait_count = 70;

	if(rcvr_pid > 0)		/* if we have forked a rcvr process */
	{
		xmtr_killed_rcvr = 1;
		rcvr_log_fp = NULL;
		xmtr_signals();
		kill(rcvr_pid,sig);
		if(sig != SIGUSR2)	/* rcvr does not die on SIGUSR2 */
		{
			errno = 0;
			while(wait_count)
			{
				if(kill(rcvr_pid,0) && (errno == ESRCH))
					break;
				errno = 0;
				Nap(40L);
				wait_count--;
			}
			if(!wait_count)
			{
				while(!kill(rcvr_pid,SIGKILL))
				{
					wait((int *)0);
					Nap(40L);
				}
			}
			rcvr_pid = -1;
			if(rcvr_log && rcvr_log_file[0])
				rcvr_log_fp = fopen(rcvr_log_file,"a");
		}
	}

}	/* end of kill_rcvr_process */

/*+-------------------------------------------------------------------------
	termecu_code_text(code)
--------------------------------------------------------------------------*/
char *
termecu_code_text(code)
int code;
{
static char errant[16];
char *signal_name_text();

	if((code >= TERMECU_SIG1) && (code <= TERMECU_SIGN))
		return(signal_name_text(code));

	switch(code)
	{
		case TERMECU_LINE_READ_ERROR: return("line read error");
		case TERMECU_XMTR_WRITE_ERROR: return("line write error");
		case TERMECU_XMTR_FATAL_ERROR: return("detected XMTR FATAL ERROR");
		case TERMECU_RCVR_FATAL_ERROR: return("detected RCVR FATAL ERROR");
		case TERMECU_BSD4_IOCTL: return("BSD4 ioctl error");
		case TERMECU_SHM_ABL: return("SHM ABL error");
		case TERMECU_SHM_RTL: return("SHM RTL error");
		case TERMECU_NO_FORK_FOR_RCVR: return("can't fork for RCVR");
		case TERMECU_TTYIN_READ_ERROR: return("keyboard read error");
		case TERMECU_LINE_OPEN_ERROR: return("line open error");
		case TERMECU_PWENT_ERROR: return("password entry error");
		case TERMECU_USAGE: return("usage");
		case TERMECU_INIT_PROC_ERROR: return("error during initial procedure");
		case TERMECU_CONFIG_ERROR: return("configuration error");
		case TERMECU_CURSES_ERROR: return("error in curses use");
		case TERMECU_MALLOC: return("critical memory allocation failure");
		default:
			sprintf(errant,"code %u?",code);
			return(errant);
	}

}	/* end of termecu_code_text */

/*+-----------------------------------------------------------------------
	termecu(sig) -- terminate program (with comm line cleanup)

  close comm line
  return any ungetty'd line
  return user's console to normal status
  unlink shm name
  terminate program
  see termecu.h
------------------------------------------------------------------------*/
void
termecu(sig)
int sig;
{
int isig;
int save_errno = errno;
extern char initial_procedure[];

	shm->terminating = 1;			/* tell friends goodbye */
	if(xmtr_pid == getpid())		/* if we are xmtr */
	{
		for(isig = 1; isig < NSIG; isig++)
			signal(isig,SIG_IGN);
		kill_rcvr_process(SIGUSR1);
		if(shm)
		{
			if(shm->Lmodem_off_hook)
				DCE_hangup();
			if(shm->Liofd != -1)
				lclose();			/* close line */
		}

		/* make SURE we release any line acquired from getty */
		ungetty_return_line();	/* lclose() calls this via unlock_tty(),
								 * but ok to make sure
								 */

		if(windows_active)
			windows_end_signal();
		tcap_curbotleft();
		tcap_eeod();
		ttymode(0);			/* normal tty status */
		shm_done();
		if(sig > NSIG)
		{
		char s64[64];
			setcolor(colors_error);
			if(sig == TERMECU_INIT_PROC_ERROR)
				pprintf("initial procedure '%s' failed\n",initial_procedure);
			else if((sig > TERMECU_INIT_PROC_ERROR) &&
					(sig <= TERMECU_INIT_PROC_ERROR + 32))
			{
				pprintf("procedure command: exit %d\n",
					sig - TERMECU_INIT_PROC_ERROR);
			}
			else
			{
				sprintf(s64,"## XMTR %s, errno = %d",termecu_code_text(sig),
					save_errno);
				pputs(s64);
				pputs("\n");
				if(lopen_err_str[0])
				{
					pputs(lopen_err_str);
					pputs("\n");
				}
				ecu_log_event(getpid(),s64);
				errno = save_errno;
				if(errno)
					pperror("errno may not apply, but");
			}
		}
		restore_initial_colors();
#if defined(WHT) || defined(XTERM_FRIEND)
		xterm_title("xterm",0);
#endif
	}
	else							/* we are rcvr */
	{
		if(sig > NSIG)
		{
		char s64[64];
			sprintf(s64,"## RCVR %s, errno = %d",termecu_code_text(sig),
				save_errno);
			setcolor(colors_error);
			pputs(s64);
			pputs("\n");
			ecu_log_event(getpid(),s64);
			errno = save_errno;
			if(errno)
				pperror("errno may not apply, but");
		}
		restore_initial_colors();
		kill(xmtr_pid,SIGHUP);
	}
	exit(sig);
	/*NOTREACHED*/

}	/* end of termecu */

void
rcvr_common_signal_handler()
{
extern int rcvr_log;
extern int rcvr_log_raw;
extern FILE *rcvr_log_fp;

	if(rcvr_log)
	{
		if(!rcvr_log_raw)
			fputs("\n",rcvr_log_fp);
		fclose(rcvr_log_fp);
	}

	exit(0);
}
void
rcvr_SIGTERM_handler()
{
	rcvr_common_signal_handler();
}
void
rcvr_SIGINT_handler()
{
	signal(SIGINT,rcvr_SIGINT_handler);
}
void
rcvr_SIGUSR1_handler()
{
	rcvr_common_signal_handler();
}
void
rcvr_SIGUSR2_handler()
{
	signal(SIGUSR2,rcvr_SIGUSR2_handler);
	shmr_process_rcvr_SIGUSR2();
}
void
rcvr_SIGHUP_handler()
{
	rcvr_common_signal_handler();
}
/*+-------------------------------------------------------------------------
	rcvr_death_handler(sig)
--------------------------------------------------------------------------*/
void
rcvr_death_handler(sig)
int sig;
{
	int itmp;
#ifdef lint
	int *open_elevator_shaft = (int *)0;
#else
	int *open_elevator_shaft = (int *)(shm - 1);
#endif

	ttymode(0);
	fprintf(stderr,"\r\nreceiver process caught signal %s\r\n",
	    signal_name_text(sig));
	fprintf(stderr,"cursor y=%d x=%d\r\n",shm->cursor_y,shm->cursor_x);
	signal(SIGSEGV,SIG_DFL);
	signal(SIGBUS,SIG_DFL);
	itmp = *open_elevator_shaft;
	_exit(-1);
}	/* end of rcvr_death_handler */

/*+-------------------------------------------------------------------------
	xmtr_SIGINT_handler()
--------------------------------------------------------------------------*/
void
xmtr_SIGINT_handler()
{
	if(ttymode_termecu_on_sigint)
		termecu(SIGINT);

	signal(SIGINT,xmtr_SIGINT_handler);
	sigint = 1;
	proc_interrupt = 1;
}	/* end of xmtr_SIGINT_handler */

void
xmtr_SIGHUP_handler(sig)
int sig;
{
	termecu(sig);
}
void
xmtr_SIGTERM_handler(sig)
int sig;
{
	termecu(sig);
}

void
xmtr_SIGUSR2_handler()
{
	signal(SIGUSR2,xmtr_SIGUSR2_handler);
	shmx_process_xmtr_SIGUSR2();
}

/*+-------------------------------------------------------------------------
	xmtr_death_handler(sig)
--------------------------------------------------------------------------*/
void
xmtr_death_handler(sig)
int sig;
{
int itmp;
#ifdef lint
int *open_elevator_shaft = (int *)0;
#else
int *open_elevator_shaft = (int *)(shm - 1);
#endif

	ttymode(0);
	fprintf(stderr,"\r\ntransmitter process caught signal %s\n",
		signal_name_text(sig));
	for(itmp = 1; itmp < NSIG; itmp++)
		signal(itmp,SIG_IGN);
	kill_rcvr_process(SIGUSR1);
	signal(SIGSEGV,SIG_DFL);
	signal(SIGBUS,SIG_DFL);
	itmp = *open_elevator_shaft;
	printf("itmp=%d oes=%08lx\n",itmp,open_elevator_shaft);
	_exit(-1);
}	/* end of xmtr_death_handler */

/*+-------------------------------------------------------------------------
	xmtr_SIGCLD_handler()
--------------------------------------------------------------------------*/
void
xmtr_SIGCLD_handler()
{
extern int xmtr_killed_rcvr;

WAIT:
	errno = 0;
	if((last_child_wait_pid = wait(&last_child_wait_status)) < 0)
	{
		if(errno == EINTR)
			goto WAIT;
	}

#if defined(FORK_DEBUG)
	sprintf(s40,"DEBUG fork SIGCLD pid %d term %x",
		last_child_wait_pid,last_child_wait_status);
	ecu_log_event(getpid(),s40);		/* xmtr_SIGCLD_handler() */
#endif

	if((last_child_wait_pid  == rcvr_pid) && !xmtr_killed_rcvr)
	{
		fprintf(stderr,"\r\nECU receiver process died unexpectedly\r\n");
		termecu(TERMECU_RCVR_FATAL_ERROR);
	}
	signal(SIGCLD,xmtr_SIGCLD_handler);

}	/* end of xmtr_SIGCLD_handler */

/*+-------------------------------------------------------------------------
	child_signals() - signal() calls for children processes
--------------------------------------------------------------------------*/
void
child_signals()
{
	signal(SIGHUP,SIG_DFL);
	signal(SIGINT,SIG_DFL);
	signal(SIGTERM,SIG_DFL);
	signal(SIGQUIT,SIG_DFL);
	signal(SIGUSR1,SIG_DFL);
	signal(SIGUSR2,SIG_DFL);
	signal(SIGCLD,SIG_DFL);

}	/* end of child_signals */

/*+-------------------------------------------------------------------------
	xmtr_signals()
--------------------------------------------------------------------------*/
void
xmtr_signals()
{

	signal(SIGHUP,xmtr_SIGHUP_handler);
	signal(SIGQUIT,SIG_IGN);
	signal(SIGINT,xmtr_SIGINT_handler);
	signal(SIGTERM,xmtr_SIGTERM_handler);
	signal(SIGCLD,xmtr_SIGCLD_handler);
	signal(SIGUSR2,xmtr_SIGUSR2_handler);
#ifdef SIGILL
	signal(SIGILL,xmtr_death_handler);
#endif
#ifdef SIGIOT
	signal(SIGIOT,xmtr_death_handler);
#endif
#ifdef SIGABRT
	signal(SIGABRT,xmtr_death_handler);
#endif
#ifdef SIGEMT
	signal(SIGEMT,xmtr_death_handler);
#endif
#ifdef SIGFPE
	signal(SIGFPE,xmtr_death_handler);
#endif
#ifdef SIGBUS
	signal(SIGBUS,xmtr_death_handler);
#endif
#ifdef SIGSEGV
	signal(SIGSEGV,xmtr_death_handler);
#endif

}	/* end of xmtr_signals */

/*+-------------------------------------------------------------------------
	rcvr_signals()
--------------------------------------------------------------------------*/
void
rcvr_signals()
{
	signal(SIGHUP,rcvr_SIGHUP_handler);
	signal(SIGQUIT,SIG_IGN);
#ifdef RCVR_CLEAN_CORE_DUMP
	signal(SIGILL,SIG_DFL);
	signal(SIGIOT,SIG_DFL);
	signal(SIGABRT,SIG_DFL);
	signal(SIGEMT,SIG_DFL);
	signal(SIGFPE,SIG_DFL);
	signal(SIGBUS,SIG_DFL);
	signal(SIGSEGV,SIG_DFL);
#else
#ifdef SIGILL
	signal(SIGILL,rcvr_death_handler);
#endif
#ifdef SIGIOT
	signal(SIGIOT,rcvr_death_handler);
#endif
#ifdef SIGABRT
	signal(SIGABRT,rcvr_death_handler);
#endif
#ifdef SIGEMT
	signal(SIGEMT,rcvr_death_handler);
#endif
#ifdef SIGFPE
	signal(SIGFPE,rcvr_death_handler);
#endif
#ifdef SIGBUS
	signal(SIGBUS,rcvr_death_handler);
#endif
#ifdef SIGSEGV
	signal(SIGSEGV,rcvr_death_handler);
#endif
#endif
	signal(SIGTERM,rcvr_SIGTERM_handler);
	signal(SIGINT,rcvr_SIGINT_handler);
	signal(SIGUSR1,rcvr_SIGUSR1_handler);
	signal(SIGUSR2,rcvr_SIGUSR2_handler);

}	/* end of rcvr_signals */

/* vi: set tabstop=4 shiftwidth=4: */
