#ifndef lint
char *revision = "ecuungetty 3.13";
#endif
/*+-------------------------------------------------------------------------
	ecuungetty.c - ecu "ungetty" program
	wht@n4hgf.Mt-Park.GA.US

Get a line:
ecuungetty /dev/ttyxx <bamboozle-str>
ecuungetty -g /dev/ttyxx <bamboozle-str>

Test a line's atatus:
ecuungetty -t /dev/ttyxx <bamboozle-str>

Return a line:
ecuungetty -r /dev/ttyxx <bamboozle-str>

  Defined functions:
	ecu_log_event(pid,logstr)
	errno_text(errnum)
	hangup()
	main(argc,argv,envp)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-10-1991-17:39-wht@n4hgf-US_WEGOIT handling */
/*:08-07-1991-14:15-wht@n4hgf-add debug log event code */
/*:07-25-1991-12:57-wht@n4hgf-ECU release 3.10 */
/*:09-19-1990-19:36-wht@n4hgf-ecu_log_event now gets pid for log from caller */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <utmp.h>
#include <sys/locking.h>
#include "../stdio_lint.h"
#include "../ecuungetty.h"
#include "../utmpstatus.h"

extern struct utmp last_utmp;

int xmtr_pid;

/*+-------------------------------------------------------------------------
	ecu_log_event(pid,event_note)
--------------------------------------------------------------------------*/
void
ecu_log_event(pid,event_note)
int pid;
char *event_note;
{
#if defined(WHT) && defined(SHARE_DEBUG)
FILE *ecu_log_fp;
static char logname[128] = "/u1/wht/.ecu/log";

	if((ecu_log_fp = fopen(logname,"a")) != NULL)
	{
		locking(fileno(ecu_log_fp),LK_LOCK,0L);
		fputs("ECUUNGET",ecu_log_fp);
		fprintf(ecu_log_fp,"-%05d-(%05d) ",getppid(),pid);
		fputs(event_note,ecu_log_fp);
		fputs("\n",ecu_log_fp);
		fflush(ecu_log_fp);
		locking(fileno(ecu_log_fp),LK_UNLCK,0L);
		fclose(ecu_log_fp);
	}
#endif
}	/* end of ecu_log_event */

/*+-------------------------------------------------------------------------
	termecu() - "dummy" for utmpstat.c
This particular incantation will only be called if utmp is non-existent
or not readable.......
--------------------------------------------------------------------------*/
termecu()
{
	exit(UGE_NOTROOT);
}	/* end of hangup */

/*+-------------------------------------------------------------------------
	errno_text(errnum)
--------------------------------------------------------------------------*/
char *
errno_text(errnum)
int errnum;
{
static char errstr[12];
	sprintf(errstr,"%d",errnum);
	return(errstr);
}	/* end of errno_text */

/*+-------------------------------------------------------------------------
	main(argc,argv,envp)
--------------------------------------------------------------------------*/
main(argc,argv,envp)
int argc;
char **argv;
char **envp;
{
int op = 'g';	/* assume "get" operation */
int status;
int itmp;
char *tty = argv[1];
char *bamboozlement = argv[2];
char *bamboozle();

	if(geteuid() != 0)
		exit(UGE_NOTROOT);

	if(*argv[1] == '-')
	{
		switch(op = *(argv[1] + 1))
		{
			case 'r':
			case 't':
				break;
			default:
				exit(UGE_BADSWITCH);
		}
		if(argc < 3)
			exit(UGE_BADARGC);
		tty = argv[2];
		bamboozlement = argv[3];
	}
	else if(argc <= 2)
		exit(UGE_BADARGC);

	if(getuid() != 0)
	{
		if(strcmp(bamboozlement,bamboozle(getppid())))
			exit(UGE_CALLER);
	}

	xmtr_pid = getppid();
	status = utmp_status(tty);
#if defined(WHT) && defined(SHARE_DEBUG)
	{ char s128[128];
		sprintf(s128,"-%c utmp status=%d",op,status);
		ecu_log_event(getpid(),s128);
	}
#endif

	switch(op)
	{
		case 'g':
			switch(status)
			{
				case US_NOTFOUND:	/* not in utmp, or getty dead */
					exit(UG_NOTENAB);
				case US_LOGIN:		/* enabled for login, idle */
					kill(last_utmp.ut_pid,SIGUSR1);
					nap(200L);
					exit(UG_ENAB);
				case US_DIALOUT:	/* enabled for login, currently dialout */
				case US_LOGGEDIN:	/* enabled for login, in use */
					exit(UG_FAIL);
				case US_WEGOTIT:	/* we on it */
					if(!kill(last_utmp.ut_pid))	/* is there a getty? */
						exit(UG_ENAB);			/* yes */
					else
						exit(UG_NOTENAB);
			}
			break;

		case 't':
			switch(status)
			{
				case US_NOTFOUND:	/* not in utmp, or getty dead */
					exit(UGE_T_NOTFOUND);
				case US_LOGIN:		/* enabled for login, idle */
					exit(UGE_T_LOGIN);
				case US_LOGGEDIN:	/* enabled for login, in use */
					exit(UGE_T_LOGGEDIN);
				case US_WEGOTIT:	/* we have he line */
				case US_DIALOUT:	/* enabled for login, currently dialout */
					exit(UG_RESTART);
			}
			break;

		case 'r':
			switch(status)
			{
				case US_NOTFOUND:	/* not in utmp, or getty dead */
				case US_LOGIN:		/* enabled for login, idle */
				case US_LOGGEDIN:	/* enabled for login, in use */
					exit(0);
				case US_WEGOTIT:	/* we own it */
				case US_DIALOUT:	/* enabled for login, currently dialout */
					itmp = 5;
					while(itmp--)
					{
						if(kill(last_utmp.ut_pid,SIGUSR2))
							break;
						nap(100L);
					}
					exit(0);
			}
			break;
	}
	exit(0);
}	/* end of main */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of ecuungetty.c */
